% integrate double layer

% Integrate 
%   dx/d(psi)= (2RT/epw * SUM c_i*[exp(-z_i F psi / RT) - 1])^-.5;

clear global;
global Na_0    K_0    Mg_0    Ca_0    Cl_0   RT   F   epw;
global dha_Na  dha_K  dha_Mg  dha_Ca  dha_Cl;

%     J/kmol         C/kmol          e_w    e_0
RT = 2477.57e3; F = 96485.e3; epw = 78.5 * 8.854e-12;

% closest distance to surface = Debye-Huckel a ...
dha_Na = 4.0e-10;  dha_K = 3.501e-10; dha_Mg = 5.5e-10; dha_Ca = 4.5e-10; dha_Cl = 3.5e-10;

% conc's in kmol/m3 at x = inf, psi = 0 ...
Na_0 = 0.0037; K_0 = 0.0003; Mg_0 = 0.0005;  Ca_0 = 0.0025;  Cl_0 = 0.01;

if Cl_0 == 0, dha_Cl = 1e4; end
if Na_0 == 0, dha_Na = 1e4; end
if K_0  == 0, dha_K  = 1e4; end 
if Mg_0 == 0, dha_Mg = 1e4; end
if Ca_0 == 0, dha_Ca = 1e4; end

function x_psidot = inv_dl(psi1, x1);

  % dx/d(psi) = (2RT/epw * SUM c_i*[exp(-z_i F psi / RT) - 1])^-.5
  
  global Na_0    K_0    Mg_0    Ca_0    Cl_0   RT   F   epw;
  global dha_Na  dha_K  dha_Mg  dha_Ca  dha_Cl;
  
  d = exp(-F*psi1/RT);
  % Na
  if x1 >= dha_Na
          d1 = Na_0*(d - 1);
  else d1 = 0; end;
  % K
  if x1 >= dha_K
          d2 = K_0*(d - 1);
  else d2 = 0; end;
  % Mg
  if x1 >= dha_Mg
          d3 = Mg_0*(d*d - 1);
  else d3 = 0; end;
  % Ca
  if x1 >= dha_Ca
          d4 = Ca_0*(d*d - 1);
  else d4 = 0; end;
  % Cl
  if x1 >= dha_Cl
          d5 = Cl_0*(1/d - 1);
  else d5 = 0; end;
  
  dall = d1 + d2 + d3 + d4 + d5;
  
  if dall == 0  dall = 1e-21; end;
  x_psidot = (2 * RT / epw * abs(dall))^(-0.5);
endfunction

min1 = min([dha_Na, dha_K, dha_Mg, dha_Ca, dha_Cl]);
max1 = max([dha_Na, dha_K, dha_Mg, dha_Ca, dha_Cl]);

vopt = odeset ("InitialStep", 1e-12, "Abstol", 1e-15, "RelTol", 1e-6);
% Cl_0 = 0.01                   psi_0  psi_end     x_0
[psi1, x1] = ode45('inv_dl', [-9.137e-2   -1.e-5], [dha_Cl 40e-9], vopt);

l = length(x1)
x(1) = x1(1); psi(1) = psi1(1); a = x1(1); i = 2;
for j = 2: l
  if x1(j) < 10e-10, if x1(j) - a > 1e-13
    a = x1(j); x(i) = x1(j); psi(i) = psi1(j); i += 1; end
  else, if x1(j) - a > 1e-11 
    a = x1(j); x(i) = x1(j); psi(i) = psi1(j); i += 1; end
end, end;

% figure(1);  clf;  hold off;
% xlabel('Distance from charged surface / nm');
% ylabel('Potential / V');
% hold on;  axis([0, 20, -0.1, 0]);
% plot(1e9*x, psi, '-');

l = length(x);
Na = zeros(l,1); K = zeros(l,1); Mg = zeros(l,1); Ca = zeros(l,1); Cl = zeros(l,1);

d = exp(-F*psi/RT);

% Fill vector ...
Na = Na_0 * d;    K = K_0 * d;
Mg = Mg_0 * d.*d; Ca = Ca_0 * d.*d;
Cl = Cl_0 ./ d;

for i = 1:l, if x(i) < max1
        if x(i) < dha_Na, Na(i) = 0; end;
        if x(i) < dha_K , K(i)  = 0; end;
        if x(i) < dha_Mg, Mg(i) = 0; end;
        if x(i) < dha_Ca, Ca(i) = 0; end;
        if x(i) < dha_Cl, Cl(i) = 0; end;
end, end;

% integrate to kmol / m2 ...
if Cl_0 > 0, dl_Cl = quadgk(@(xi) interp1(x, Cl, xi), min1, x(l));  else, dl_Cl = 0; end
if Na_0 > 0, dl_Na = quadgk(@(xi) interp1(x, Na, xi), min1, x(l));  else, dl_Na = 0; end 
if K_0  > 0, dl_K  = quadgk(@(xi) interp1(x, K , xi), min1, x(l));  else, dl_K  = 0; end
if Mg_0 > 0, dl_Mg = quadgk(@(xi) interp1(x, Mg, xi), min1, x(l));  else, dl_Mg = 0; end
if Ca_0 > 0, dl_Ca = quadgk(@(xi) interp1(x, Ca, xi), min1, x(l));  else, dl_Ca = 0; end

printf("dl_Na = %g kmol/m2, dl_Ca = %g kmol/m2\n, dl_Cl = %g kmol/m2\n", dl_Na, dl_Ca, dl_Cl)
% in mol_charge ...
psi_0 = psi(1)
Coul_m2 = F * (-dl_Na - dl_K - 2*(dl_Mg + dl_Ca) + dl_Cl) * 1e3
charge = Coul_m2 / F; printf("Charge = %g eq/m2\n", charge) ;

b_Na = -1e3*dl_Na/charge; 
if K_0  > 0, b_K = -1e3*dl_K/charge;   K_K = b_Na/b_K*K_0/Na_0,               end
if Mg_0 > 0, b_Mg = -2e3*dl_Mg/charge; K_Mg = b_Na/b_Mg^(0.5)*Mg_0^(0.5)/Na_0 end
if Ca_0 > 0, b_Ca = -2e3*dl_Ca/charge; K_Ca = b_Na/b_Ca^(0.5)*Ca_0^(0.5)/Na_0 end

% printf("Bb_Ca = %g, Aa_Ca = %g\n", b_Ca, 2 * Ca_0 / Cl_0) ;

b_Cl =  1e3 * dl_Cl / charge
f_dl_Na = -1e3 * dl_Na / charge
f_dl_Ca = -2e3 * dl_Ca / charge

figure(2);
clf;
hold off;
xlabel('Distance from charged surface / nm');
ylabel('mol/L');
hold on;

 axis([0, 2, 0, 0.5]);
if Cl_0 > 0, plot(1e9*x, 30*Cl, 'k:','linewidth',1.);   end
if Na_0 > 0, plot(1e9*x, Na, '-','linewidth',1.1);     end
if K_0  > 0, plot(1e9*x, 5*K, '-');                    end
if Mg_0 > 0, plot(1e9*x, 1*Mg, ':');                   end
if Ca_0 > 0, plot(1e9*x, Ca/5, '- -','linewidth',1.2); end


if Cl_0 > 0, text(1.5, 0.1, 'Cl * 30');  end %, 'fontangle', 'ital', 'fontunderl', 'off')
if Ca_0 > 0, text(0.57, 0.42, 'Ca / 5'); end %, 'fontangle', 'ital', 'fontunderl', 'off')
if Na_0 > 0, text(0.3, 0.14, 'Na');      end  %, 'fontangle', 'ital', 'fontunderl', 'off')
if K_0  > 0, text(0.17, 0.06, 'K * 5');  end  %, 'fontangle', 'ital', 'fontunderl', 'off')
if Mg_0 > 0, text(0.62, 0.29, 'Mg');     end  %, 'fontangle', 'ital', 'fontunderl', 'off')

